@@ -1,6 +1,6 @@
 # This file is part of the Astrometry.net suite.
 # Licensed under a 3-clause BSD style license - see LICENSE
-from plotstuff_c import *
+from astrometry.blind.plotstuff_c import *
 
 # Could consider using swig's "addmethods" mechanism to create this "class" rep.
 
@@ -14,12 +14,14 @@ class Plotstuff(object):
                   
 
     def __init__(self, outformat=None, size=None, ra=None, dec=None, width=None,
-                 rdw=None, wcsfn=None, wcsext=0, alpha=1.):
+                 rdw=None, wcsfn=None, wcsext=0, alpha=1., outfn=None):
         p = plotstuff_new()
         self.pargs = p
         if outformat is not None:
             outformat = Plotstuff.format_map.get(outformat, outformat)
             self.outformat = outformat
+        if outfn is not None:
+            self.outfn = outfn
         if size is not None:
             self.size = size
             self.color = 'black'
