@@ -582,8 +582,11 @@ static void after_solved(augment_xylist_t* axy,
 
     // create new FITS file...
     if (axy->fitsimgfn && sf->newfitsfn && file_exists(axy->wcsfn)) {
+        int ext = axy->isfits ? axy->fitsimgext : 0;
         logmsg("Creating new FITS file \"%s\"...\n", sf->newfitsfn);
-        if (new_wcs(axy->fitsimgfn, axy->wcsfn, sf->newfitsfn, TRUE)) {
+        //logmsg("From image %s, ext
+        if (new_wcs(axy->fitsimgfn, ext, axy->wcsfn,
+                    sf->newfitsfn, TRUE)) {
             ERROR("Failed to create FITS image with new WCS headers");
             exit(-1);
         }
