@@ -18,6 +18,7 @@
 # along with the Astrometry.net suite ; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
+from __future__ import print_function
 
 import urllib
 import os
@@ -25,9 +26,9 @@ import os
 prefix = 'http://cdsarc.u-strasbg.fr/ftp/cats/aliases/U/UCAC3/UCAC3/'
 
 for i in range(1, 361):
-	name = 'z'+str(i).zfill(3)+'.bz2'
-	url_name = prefix+name
+    name = 'z'+str(i).zfill(3)+'.bz2'
+    url_name = prefix+name
 
-	print 'Downloading: '+url_name
-	ucac_file = urllib.URLopener()
-	ucac_file.retrieve(url_name, name)
+    print('Downloading: '+url_name)
+    ucac_file = urllib.URLopener()
+    ucac_file.retrieve(url_name, name)
