@@ -48,7 +48,7 @@ setup(cmdclass={'build_ext': an_build_ext},
       version = '1.0',
       description = '',
       author = 'Astrometry.net (Dustin Lang)',
-      author_email = 'dstn@cmu.edu',
+      author_email = 'dstndstn@gmail.com',
       url = 'http://astrometry.net',
       py_modules = [ 'util' ],
       ext_modules = [c_swig_module])
